@@ -12,7 +12,6 @@
 #                                                             #
 ###############################################################
 
-
 setGeneric("layers", 
            function(x) standardGeneric("layers"))
 
@@ -21,7 +20,7 @@ setGeneric("coverages",
 
 setGeneric("bands", 
            function(x) standardGeneric("bands"))
-           
+
 setGeneric("is.twdtwTimeSeries", 
            function(x) standardGeneric("is.twdtwTimeSeries"))
 
@@ -30,18 +29,21 @@ setGeneric("is.twdtwMatches",
 
 setGeneric("is.twdtwRaster", 
            function(x) standardGeneric("is.twdtwRaster"))
-           
+
+setGeneric("projecttwdtwRaster", 
+           function(x, ...) standardGeneric("projecttwdtwRaster"))
+
 as.list.twdtwTimeSeries = function(x) lapply(seq_along(x), function(i) 
-      new("twdtwTimeSeries", x[[i]], labels(x)[i]) )
+  new("twdtwTimeSeries", x[[i]], labels(x)[i]) )
 
 as.list.twdtwRaster = function(x) {
-    I = coverages(x)
-    names(I) = I
-    lapply(I, function(i) x[[i]])
-  }
+  I = coverages(x)
+  names(I) = I
+  lapply(I, function(i) x[[i]])
+}
 
 as.list.twdtwMatches = function(x) lapply(seq_along(x@timeseries), function(i) 
-      new("twdtwMatches", new("twdtwTimeSeries", x@timeseries[[i]], labels(x@timeseries)[i]), x@patterns, list(x@alignments[[i]])) )
+  new("twdtwMatches", new("twdtwTimeSeries", x@timeseries[[i]], labels(x@timeseries)[i]), x@patterns, list(x@alignments[[i]])) )
 
 dim.twdtwTimeSeries = function(x){
   res = data.frame(as.character(labels(x)), t(sapply(x@timeseries, dim)))
@@ -178,7 +180,7 @@ setMethod(f = "ncol", "twdtwRaster",
 #' @export
 setMethod(f = "nrow", "twdtwRaster",
           definition = nrow.twdtwRaster)
-          
+
 #' @inheritParams twdtwRaster-class
 #' @rdname twdtwRaster-class
 #' @export
@@ -204,26 +206,26 @@ setMethod(f = "layers", "twdtwRaster",
 #' @export
 setMethod(f = "coverages", "twdtwRaster",
           definition = coverages.twdtwRaster)
-          
+
 #' @aliases bands
 #' @inheritParams twdtwRaster-class
 #' @rdname twdtwRaster-class
 #' @export
 setMethod(f = "bands", "twdtwRaster",
           definition = bands.twdtwRaster)
-         
+
 #' @inheritParams twdtwRaster-class
 #' @rdname twdtwRaster-class
 #' @export
 setMethod(f = "names", "twdtwRaster",
           definition = names.twdtwRaster)
-          
+
 #' @inheritParams twdtwRaster-class
 #' @rdname twdtwRaster-class
 #' @export
 setMethod(f = "index", "twdtwRaster",
           definition = index.twdtwRaster)
-   
+
 #' @inheritParams twdtwTimeSeries-class
 #' @rdname twdtwTimeSeries-class
 #' @export
@@ -235,13 +237,13 @@ setMethod(f = "index", "twdtwTimeSeries",
 #' @export
 setMethod(f = "index", "twdtwMatches",
           definition = index.twdtwMatches)
-          
+
 #' @inheritParams twdtwTimeSeries-class
 #' @rdname twdtwTimeSeries-class
 #' @export
 setMethod(f = "nrow", "twdtwTimeSeries",
           definition = nrow.twdtwTimeSeries)
-          
+
 #' @inheritParams twdtwTimeSeries-class
 #' @rdname twdtwTimeSeries-class
 #' @export
@@ -253,7 +255,7 @@ setMethod(f = "ncol", "twdtwTimeSeries",
 #' @export
 setMethod(f = "length", signature = signature("twdtwRaster"), 
           definition = length.twdtwRaster)
-          
+
 #' @inheritParams twdtwTimeSeries-class
 #' @rdname twdtwTimeSeries-class
 #' @export
@@ -309,7 +311,9 @@ setMethod("[", "twdtwRaster", function(x, i) {
   if(any(i > nlayers(x)))
     stop("subscript out of bounds")
   if(any(is.na(i))) stop("NA index not permitted")
-  new("twdtwRaster", timeseries=x@timeseries[i], timeline = x@timeline, doy = x@timeseries[[1]])
+  if(any("doy"==layers(x)))
+    return(new("twdtwRaster", timeseries=x@timeseries[i], timeline = x@timeline, doy = x@timeseries[[1]]))
+  new("twdtwRaster", timeseries=x@timeseries[i], timeline = x@timeline)
 })
 
 #' @inheritParams twdtwRaster-class
@@ -330,17 +334,20 @@ setMethod("[[", "twdtwRaster", function(x, i) {
 setMethod("[", "twdtwMatches", function(x, i, j, drop=TRUE) {
   if(length(x@alignments)<1) return(x@alignments)
   if(missing(i)) i = 1:length(x@alignments)
+  # if(missing(j)) j = 2:length(x@patterns)
   if(any(is.na(i))) stop("NA index not permitted")
+  if(class(i)=="character") i = match(i, names(x@timeseries@timeseries))
   res = x@alignments[i]
   if(missing(j)) j = 1:length(res[[1]])
+  if(class(j)=="character") j = match(j, names(x@patterns@timeseries))
   if(any(is.na(j))) stop("NA index not permitted")
   res = lapply(res, function(x) x[j])
   res = res[sapply(res, length)>0]
   if(!drop) return(res)
   lapply(res, function(x){
     res = do.call("rbind", lapply(seq_along(x), function(jj){
-            data.frame(Alig.N=seq_along(x[[jj]]$distance),from=x[[jj]]$from, to=x[[jj]]$to, distance=x[[jj]]$distance, label=x[[jj]]$label, row.names=NULL)
-          }))
+      data.frame(Alig.N=seq_along(x[[jj]]$distance),from=x[[jj]]$from, to=x[[jj]]$to, distance=x[[jj]]$distance, label=x[[jj]]$label, row.names=NULL)
+    }))
     res[order(res$from),]      
   })
 })
@@ -358,20 +365,20 @@ setMethod("[[", c("twdtwMatches", "numeric"), function(x, i, j,drop=TRUE) {
 #' @rdname twdtwTimeSeries-class
 #' @export
 setMethod("labels", signature = signature(object="twdtwTimeSeries"),
-          definition = function(object) object@labels)
+          definition = function(object) as.character(object@labels))
 
 #' @inheritParams twdtwTimeSeries-class
 #' @rdname twdtwTimeSeries-class
 #' @export
 setMethod("levels", "twdtwTimeSeries",
-          definition = function(x) levels(labels(x)))
+          definition = function(x) levels(factor(labels(x))))
 
 #' @inheritParams twdtwRaster-class
 #' @rdname twdtwRaster-class
 #' @export
 setMethod("labels", signature = signature(object="twdtwRaster"),
           definition = function(object) as.character(object@labels))
-          
+
 #' @inheritParams twdtwMatches-class
 #' @rdname twdtwMatches-class
 #' @export
@@ -390,18 +397,29 @@ setMethod("labels",
 setMethod("crop", 
           signature = signature("twdtwRaster"),
           definition = function(x, y, ...){
-            x@timeseries = lapply(x@timeseries, crop, y=extent(y), ...)
+            x@timeseries = lapply(x@timeseries, crop, y=y, ...)
             x
           }
 )
 
+#' @inheritParams twdtwRaster-class
+#' @rdname twdtwRaster-class
+#' @param obj object of cless twdtwRaster 
+#' @export
+setMethod("coordinates", 
+          signature = signature("twdtwRaster"),
+          definition = function(obj, ...){
+            coordinates(obj@timeseries[[1]], ...)
+          }
+)
+
 #' @inheritParams twdtwRaster-class
 #' @rdname twdtwRaster-class
 #' @export
 setMethod("extent", 
           signature = signature("twdtwRaster"),
           definition = function(x, y, ...){
-            extent(x@timeseries$doy)
+            extent(x@timeseries[[1]])
           }
 )
 
@@ -436,6 +454,96 @@ show.twdtwRaster = function(object){
   invisible(NULL)
 }
 
+# Show objects of class twdtwAssessment
+show.twdtwAssessment = function(object){
+  cat("An object of class \"twdtwAssessment\"\n")
+  cat("Number of classification intervals:",length(object@accuracyByPeriod),"\n")
+  cat("Accuracy metrics summary\n")
+  cat("\nOverall\n")
+  aux = object@accuracySummary$OverallAccuracy
+  names(aux) = gsub("ci", "ci*", names(aux))
+  print(aux, digits=2)
+  cat("\nUser's\n")
+  aux = object@accuracySummary$UsersAccuracy
+  colnames(aux) = gsub("ci", "ci*", colnames(aux))
+  print(aux, digits=2)
+  cat("\nProducer's\n")
+  aux = object@accuracySummary$ProducersAccuracy
+  colnames(aux) = gsub("ci", "ci*", colnames(aux))
+  print(aux, digits=2)
+  cat("\nArea and uncertainty\n")
+  aux = object@accuracySummary$AreaUncertainty
+  colnames(aux) = gsub("ci", "ci*", colnames(aux))
+  print(aux, digits=2)
+  cat("\n*",100*object@accuracySummary$conf.int,"% confidence interval\n")
+  invisible(NULL)
+}
+
+# Show objects of class twdtwCrossValidation
+show.twdtwCrossValidation = function(object){
+  res = summary(object, conf.int=.95)
+  cat("An object of class \"twdtwCrossValidation\"\n")
+  cat("Number of data partitions:",length(object@partitions),"\n")
+  cat("Accuracy metrics using bootstrap simulation (CI .95)\n")
+  cat("\nOverall\n")
+  print(res$Overall, digits=2)
+  cat("\nUser's\n")
+  print(res$Users, digits=2)
+  cat("\nProducer's\n")
+  print(res$Producers, digits=2)
+  invisible(NULL)
+}
+
+# Project raster which belong to a twdtwRaster object 
+projecttwdtwRaster.twdtwRaster = function(x, to, ...){
+  x@timeseries = lapply(x@timeseries, projectRaster, to, ...)
+  x
+}
+
+summary.twdtwCrossValidation = function(object, conf.int=.95, ...){
+  
+  ov = do.call("rbind", lapply(object@accuracy, function(x){
+    data.frame(OV=x$OverallAccuracy, row.names = NULL)
+  }))
+  
+  uapa = do.call("rbind", lapply(object@accuracy, function(x){
+    data.frame(label=names(x$UsersAccuracy), UA=x$UsersAccuracy, PA=x$ProducersAccuracy, row.names = NULL)
+  }))
+  
+  sd_ov = sd(ov[, c("OV")])
+  sd_uapa = aggregate(uapa[, c("UA","PA")], list(uapa$label), sd)
+  l_names = levels(uapa$label)
+  names(l_names) = l_names
+  ic_ov = mean_cl_boot(x = ov[, c("OV")], conf.int = conf.int, ...)
+  names(ic_ov) = NULL
+  assess_ov = unlist(c(Accuracy=ic_ov[1], sd=sd_ov, CImin=ic_ov[2], CImax=ic_ov[3]))
+  ic_ua = t(sapply(l_names, function(i) mean_cl_boot(x = uapa$UA[uapa$label==i], conf.int = conf.int, ...)))
+  names(ic_ua) = NULL
+  assess_ua = data.frame(Accuracy=unlist(ic_ua[,1]), sd=sd_uapa[,"UA"], CImin=unlist(ic_ua[,2]), CImax=unlist(ic_ua[,3]))
+  ic_pa = t(sapply(l_names, function(i) mean_cl_boot(x = uapa$PA[uapa$label==i], conf.int = conf.int, ...)))
+  names(ic_pa) = NULL  
+  assess_pa = data.frame(Accuracy=unlist(ic_pa[,1]), sd=sd_uapa[,"PA"], CImin=unlist(ic_pa[,2]), CImax=unlist(ic_pa[,3]))
+  list(Overall=assess_ov, Users=assess_ua, Producers=assess_pa)
+}
+
+#' @inheritParams twdtwCrossValidation-class
+#' @rdname twdtwCrossValidation-class
+#' @export
+setMethod(f = "show", "twdtwCrossValidation",
+          definition = show.twdtwCrossValidation)
+
+#' @inheritParams twdtwAssessment-class
+#' @rdname twdtwAssessment-class
+#' @export
+setMethod(f = "show", "twdtwAssessment",
+          definition = show.twdtwAssessment)
+
+#' @inheritParams twdtwCrossValidation-class
+#' @rdname twdtwCrossValidation-class
+#' @export
+setMethod(f = "summary", "twdtwCrossValidation",
+          definition = summary.twdtwCrossValidation)
+
 #' @inheritParams twdtwTimeSeries-class
 #' @rdname twdtwTimeSeries-class
 #' @export
@@ -453,7 +561,7 @@ setMethod(f = "show", "twdtwMatches",
 #' @export
 setMethod(f = "show", "twdtwRaster",
           definition = show.twdtwRaster)
-           
+
 #' @aliases is.twdtwTimeSeries
 #' @inheritParams twdtwTimeSeries-class
 #' @describeIn twdtwTimeSeries Check if the object belongs to the class twdtwTimeSeries.
@@ -467,32 +575,27 @@ setMethod("is.twdtwTimeSeries", "ANY",
 #' @export
 setMethod("is.twdtwMatches", "ANY", 
           function(x) is(x, "twdtwMatches"))
-          
+
 #' @aliases is.twdtwRaster
 #' @inheritParams twdtwRaster-class
 #' @describeIn twdtwRaster Check if the object belongs to the class twdtwRaster.
 #' @export
 setMethod("is.twdtwRaster", "ANY", 
           function(x) is(x, "twdtwRaster"))
-         
-# #' @aliases summary
-# #' @inheritParams twdtwMatches-class
-# #' @describeIn twdtwMatches Summary of objects of class twdtwMatches.
-# #' @export         
-# setMethod("summary", 
-#           signature(object = "twdtwMatches"),
-#           function(object, labels=NULL, ...){
-#             summary.twdtw(object, labels=labels, ...)
-#           }
-# )
-# 
-# summary.twdtw = function(object, ...){
-#   lapply(as.list(object), function(obj){
-#     res = lapply(labels(obj)$patterns, function(l){
-#       m = subset(obj, patterns.labels=l)[[1]]
-#       c(labels=l, N.Matches=nrow(m), summary(m$distance))
-#     })
-#     data.frame(res)
-#   })
-# }
+
+#' @aliases projecttwdtwRaster
+#' @inheritParams twdtwRaster-class
+#' @describeIn twdtwRaster project twdtwRaster object.
+#' @param crs character or object of class 'CRS'. PROJ.4 description of 
+#' the coordinate reference system. For other arguments and more details see 
+#' \code{\link[raster]{projectRaster}}.
+#' 
+#' @export
+setMethod("projecttwdtwRaster", "twdtwRaster", 
+          function(x, crs, ...) projecttwdtwRaster.twdtwRaster(x, crs, ...))
+
+
+
+
+
 
