@@ -13,10 +13,10 @@
 ###############################################################
 
 
-#' @title Plotting maps
+#' @title Plotting accumulated area
 #' @author Victor Maus, \email{vwmaus1@@gmail.com}
 #' 
-#' @description Method for plotting time series of maps.
+#' @description Method for plotting time series of accumulated area.
 #' 
 #' @param x An object of class \code{\link[dtwSat]{twdtwRaster}}.
 #' @param time.levels A \link[base]{character} or \link[base]{numeric}
@@ -33,7 +33,9 @@
 #' @param class.colors a set of aesthetic values. It must have the same 
 #' length as class.levels. Default is NULL. See 
 #' \link[ggplot2]{scale_fill_manual} for details.
-#' 
+#' @param perc if TRUE shows the results in percent of area. Otherwise shows the 
+#' area in the map units or km2 for no project raster. Default is TRUE.
+#'  
 #' @return A \link[ggplot2]{ggplot} object.
 #' 
 #' @seealso 
@@ -45,8 +47,8 @@
 #'  
 #' @examples
 #' \dontrun{
-#' # Run TWDTW analysis for raster time series 
-#' patt = yearly_patterns_mt
+#' 
+#' # Create raster time series
 #' evi = brick(system.file("lucc_MT/data/evi.tif", package="dtwSat"))
 #' ndvi = brick(system.file("lucc_MT/data/ndvi.tif", package="dtwSat"))
 #' red = brick(system.file("lucc_MT/data/red.tif", package="dtwSat"))
@@ -57,31 +59,51 @@
 #' timeline = scan(system.file("lucc_MT/data/timeline", package="dtwSat"), what="date")
 #' rts = twdtwRaster(evi, ndvi, red, blue, nir, mir, timeline = timeline, doy = doy)
 #' 
-#' time_interval = seq(from=as.Date("2007-09-01"), to=as.Date("2013-09-01"), 
-#'                     by="12 month")
+#' # Read fiels samples 
+#' field_samples = read.csv(system.file("lucc_MT/data/samples.csv", package="dtwSat"))
+#' proj_str = scan(system.file("lucc_MT/data/samples_projection", 
+#'                 package="dtwSat"), what = "character")
+#' 
+#' # Split samples for training (10%) and validation (90%) using stratified sampling 
+#' library(caret) 
+#' set.seed(1)
+#' I = unlist(createDataPartition(field_samples$label, p = 0.1))
+#' training_samples = field_samples[I,]
+#' validation_samples = field_samples[-I,]
+#' 
+#' # Create temporal patterns 
+#' training_ts = getTimeSeries(rts, y = training_samples, proj4string = proj_str)
+#' temporal_patterns = createPatterns(training_ts, freq = 8, formula = y ~ s(x))
+#' 
+#' # Run TWDTW analysis for raster time series 
 #' log_fun = weight.fun=logisticWeight(-0.1,50)
-#' 
-#' r_twdtw = twdtwApply(x=rts, y=patt, weight.fun=log_fun, breaks=time_interval, 
-#'           filepath="~/test_twdtw", overwrite=TRUE, format="GTiff", mc.cores=3)
-#' 
-#' r_lucc = twdtwClassify(r_twdtw, format="GTiff")
+#' r_twdtw = twdtwApply(x=rts, y=temporal_patterns, weight.fun=log_fun, format="GTiff", 
+#'                      overwrite=TRUE)
+#'                      
+#' # Classify raster based on the TWDTW analysis 
+#' r_lucc = twdtwClassify(r_twdtw, format="GTiff", overwrite=TRUE)
 #' 
 #' plotArea(r_lucc)
 #' 
+#' plotArea(r_lucc, perc=FALSE)
+#' 
 #' }
 #' @export
-plotArea = function(x, time.levels=NULL, time.labels=NULL, class.levels=NULL, class.labels=NULL, class.colors=NULL){
-  plot(x, type="area", time.levels=time.levels, time.labels=time.labels, class.levels=class.levels, class.labels=class.labels, class.colors=class.colors)
+plotArea = function(x, time.levels=NULL, time.labels=NULL, class.levels=NULL, class.labels=NULL, class.colors=NULL, perc=TRUE){
+  plot(x, type="area", time.levels=time.levels, time.labels=time.labels, class.levels=class.levels, class.labels=class.labels, class.colors=class.colors, perc=perc)
 }
 
-.plotArea = function(x, time.levels, time.labels, class.levels, class.labels, class.colors){
-  df.map = data.frame(coordinates(x), x[], stringsAsFactors=FALSE)
-  df.map = melt(df.map, id.vars = c("x", "y"))
-  df.map$value = factor(df.map$value, levels = class.levels, labels = class.labels)
-  df.map$variable = time.labels[match(as.character(df.map$variable), time.levels)]
+.plotArea = function(x, time.levels, time.labels, class.levels, class.labels, class.colors, perc){
+
+  df.area = do.call("rbind", lapply(time.levels, .getAreaByClass, x, class.levels, class.labels))
+  df.area = data.frame(variable = as.numeric(time.labels), df.area, stringsAsFactors = FALSE)
+  names(class.colors) = names(df.area)[-1]
+  df.area = melt(df.area, "variable", value.name = "Freq", variable.name = "value")
+  df.area$Time = as.numeric(df.area$variable)
+  df.area$variable = factor(df.area$variable)
   
-  df.area = data.frame(prop.table(xtabs(~ variable + value, df.map), 1))
-  df.area$Time = as.numeric(as.character(df.area$variable))
+  if(perc)
+    df.area$Freq = df.area$Freq / (sum(df.area$Freq) / length(time.levels))
   
   x.breaks = pretty_breaks()(range(df.area$Time))
   
@@ -89,10 +111,16 @@ plotArea = function(x, time.levels=NULL, time.labels=NULL, class.levels=NULL, cl
     geom_area(position = 'stack') + 
     scale_fill_manual(name="Legend", values = class.colors) + 
     scale_x_continuous(expand = c(0.01, 0), breaks = x.breaks) + 
-    scale_y_continuous(expand = c(0, 0), labels = percent) +
     theme(legend.position = "bottom",
           panel.background = element_blank()) + 
-    ylab("Percentage of area")
+    ylab("Area")
+  
+  if(perc){
+    gp = gp + scale_y_continuous(expand = c(0, 0), labels = percent)
+  } else {
+    gp = gp + scale_y_continuous(expand = c(0, 0)) 
+  }
+
   gp 
   
 }
